package sun.security.pkcs;

import java.util.Collections;
import java.util.EnumSet;
import sun.misc.HexDumpEncoder;
import java.util.Arrays;
import java.security.cert.CertificateException;
import java.security.cert.CertPath;
import sun.security.timestamp.TimestampToken;
import java.security.cert.CertificateFactory;
import java.security.NoSuchAlgorithmException;
import java.security.AlgorithmParameters;
import java.security.PublicKey;
import sun.security.util.KeyUtil;
import java.security.InvalidKeyException;
import java.security.InvalidAlgorithmParameterException;
import java.security.ProviderException;
import sun.security.util.SignatureUtil;
import java.security.Signature;
import sun.security.x509.KeyUsageExtension;
import java.security.MessageDigest;
import java.security.cert.CertPathValidatorException;
import java.security.SignatureException;
import sun.security.util.ObjectIdentifier;
import sun.security.util.ConstraintsParameters;
import java.security.Principal;
import java.util.ArrayList;
import java.security.cert.X509Certificate;
import java.io.OutputStream;
import sun.security.util.DerOutputStream;
import sun.security.util.DerValue;
import java.io.IOException;
import sun.security.util.DerInputStream;
import sun.security.util.Debug;
import java.security.Timestamp;
import sun.security.x509.AlgorithmId;
import sun.security.x509.X500Name;
import java.math.BigInteger;
import sun.security.util.DisabledAlgorithmConstraints;
import java.security.CryptoPrimitive;
import java.util.Set;
import sun.security.util.DerEncoder;

public class SignerInfo implements DerEncoder
{
    private static final Set<CryptoPrimitive> DIGEST_PRIMITIVE_SET;
    private static final Set<CryptoPrimitive> SIG_PRIMITIVE_SET;
    private static final DisabledAlgorithmConstraints JAR_DISABLED_CHECK;
    BigInteger version;
    X500Name issuerName;
    BigInteger certificateSerialNumber;
    AlgorithmId digestAlgorithmId;
    AlgorithmId digestEncryptionAlgorithmId;
    byte[] encryptedDigest;
    Timestamp timestamp;
    private boolean hasTimestamp;
    private static final Debug debug;
    PKCS9Attributes authenticatedAttributes;
    PKCS9Attributes unauthenticatedAttributes;
    
    public SignerInfo(X500Name issuerName, BigInteger certificateSerialNumber, AlgorithmId digestAlgorithmId, AlgorithmId digestEncryptionAlgorithmId, byte[] encryptedDigest) {
        this.hasTimestamp = true;
        this.version = BigInteger.ONE;
        this.issuerName = issuerName;
        this.certificateSerialNumber = certificateSerialNumber;
        this.digestAlgorithmId = digestAlgorithmId;
        this.digestEncryptionAlgorithmId = digestEncryptionAlgorithmId;
        this.encryptedDigest = encryptedDigest;
    }
    
    public SignerInfo(X500Name issuerName, BigInteger certificateSerialNumber, AlgorithmId digestAlgorithmId, PKCS9Attributes authenticatedAttributes, AlgorithmId digestEncryptionAlgorithmId, byte[] encryptedDigest, PKCS9Attributes unauthenticatedAttributes) {
        this.hasTimestamp = true;
        this.version = BigInteger.ONE;
        this.issuerName = issuerName;
        this.certificateSerialNumber = certificateSerialNumber;
        this.digestAlgorithmId = digestAlgorithmId;
        this.authenticatedAttributes = authenticatedAttributes;
        this.digestEncryptionAlgorithmId = digestEncryptionAlgorithmId;
        this.encryptedDigest = encryptedDigest;
        this.unauthenticatedAttributes = unauthenticatedAttributes;
    }
    
    public SignerInfo(DerInputStream derInputStream) throws IOException, ParsingException {
        this(derInputStream, false);
    }
    
    public SignerInfo(DerInputStream derInputStream, boolean b) throws IOException, ParsingException {
        this.hasTimestamp = true;
        this.version = derInputStream.getBigInteger();
        DerValue[] sequence = derInputStream.getSequence(2);
        if (sequence.length != 2) {
            throw new ParsingException("Invalid length for IssuerAndSerialNumber");
        }
        this.issuerName = new X500Name(new DerValue((byte)48, sequence[0].toByteArray()));
        this.certificateSerialNumber = sequence[1].getBigInteger();
        this.digestAlgorithmId = AlgorithmId.parse(derInputStream.getDerValue());
        if (b) {
            derInputStream.getSet(0);
        }
        else if ((byte)derInputStream.peekByte() == -96) {
            this.authenticatedAttributes = new PKCS9Attributes(derInputStream);
        }
        this.digestEncryptionAlgorithmId = AlgorithmId.parse(derInputStream.getDerValue());
        this.encryptedDigest = derInputStream.getOctetString();
        if (b) {
            derInputStream.getSet(0);
        }
        else if (derInputStream.available() != 0 && (byte)derInputStream.peekByte() == -95) {
            this.unauthenticatedAttributes = new PKCS9Attributes(derInputStream, true);
        }
        if (derInputStream.available() != 0) {
            throw new ParsingException("extra data at the end");
        }
        return;
    }
    
    @Override
    public void encode(DerOutputStream derOutputStream) throws IOException {
        this.derEncode(derOutputStream);
    }
    
    public void derEncode(OutputStream outputStream) throws IOException {
        DerOutputStream derOutputStream = new DerOutputStream();
        derOutputStream.putInteger(this.version);
        DerOutputStream out = new DerOutputStream();
        this.issuerName.encode(out);
        out.putInteger(this.certificateSerialNumber);
        derOutputStream.write((byte)48, out);
        this.digestAlgorithmId.encode(derOutputStream);
        if (this.authenticatedAttributes != null) {
            this.authenticatedAttributes.encode((byte)(-96), derOutputStream);
        }
        this.digestEncryptionAlgorithmId.encode(derOutputStream);
        derOutputStream.putOctetString(this.encryptedDigest);
        if (this.unauthenticatedAttributes != null) {
            this.unauthenticatedAttributes.encode((byte)(-95), derOutputStream);
        }
        DerOutputStream derOutputStream2 = new DerOutputStream();
        derOutputStream2.write((byte)48, derOutputStream);
        outputStream.write(derOutputStream2.toByteArray());
    }
    
    public X509Certificate getCertificate(PKCS7 pkcs7) throws IOException {
        return pkcs7.getCertificate(this.certificateSerialNumber, this.issuerName);
    }
    
    public ArrayList<X509Certificate> getCertificateChain(PKCS7 pkcs7) throws IOException {
        X509Certificate certificate = pkcs7.getCertificate(this.certificateSerialNumber, this.issuerName);
        if (certificate == null) {
            return null;
        }
        ArrayList<X509Certificate> list = new ArrayList();
        list.add(certificate);
        X509Certificate[] certificates = pkcs7.getCertificates();
        if (certificates == null || certificate.getSubjectDN().equals(certificate.getIssuerDN())) {
            return list;
        }
        Principal principal = certificate.getIssuerDN();
        int i = 0;
        while (true) {
            boolean n = false;
            while (i < certificates.length) {
                if (principal.equals(certificates[i].getSubjectDN())) {
                    list.add(certificates[i]);
                    if (certificates[i].getSubjectDN().equals(certificates[i].getIssuerDN())) {
                        i = certificates.length;
                    }
                    else {
                        principal = certificates[i].getIssuerDN();
                        X509Certificate x509Certificate = certificates[i];
                        certificates[i] = x509Certificate;
                        ++i;
                    }
                    n = true;
                    break;
                }
                else {
                    ++i;
                }
            }
            if (n == 0) {
                break;
            }
            continue;
        }
        return list;
    }
    
    SignerInfo verify(PKCS7 pkcs7, byte[] array) throws NoSuchAlgorithmException, SignatureException {
        try {
            ContentInfo contentInfo = pkcs7.getContentInfo();
            if (array == null) {
                array = contentInfo.getContentBytes();
            }
            Timestamp timestamp = null;
            try {
                timestamp = this.getTimestamp();
            }
            catch (Exception ex) {}
            ConstraintsParameters o = new ConstraintsParameters(timestamp);
            String name = this.getDigestAlgorithmId().getName();
            if (this.authenticatedAttributes == null) {
            }
            else {
                ObjectIdentifier objectIdentifier = (ObjectIdentifier)this.authenticatedAttributes.getAttributeValue(PKCS9Attribute.CONTENT_TYPE_OID);
                if (objectIdentifier == null || !objectIdentifier.equals(contentInfo.contentType)) {
                    return null;
                }
                byte[] array2 = (byte[])(byte[])this.authenticatedAttributes.getAttributeValue(PKCS9Attribute.MESSAGE_DIGEST_OID);
                if (array2 == null) {
                    return null;
                }
                try {
                    SignerInfo.JAR_DISABLED_CHECK.permits(name, o);
                }
                catch (CertPathValidatorException cause) {
                    throw new SignatureException(cause.getMessage(), cause);
                }
                byte[] digest = MessageDigest.getInstance(name).digest(array);
                if (array2.length != digest.length) {
                    return null;
                }
                int i = 0;
                while (i < array2.length) {
                    if (array2[i] != digest[i]) {
                        return null;
                    }
                    ++i;
                }
                array = this.authenticatedAttributes.getDerEncoding();
            }
            String name2 = this.getDigestEncryptionAlgorithmId().getName();
            String encAlgFromSigAlg = AlgorithmId.getEncAlgFromSigAlg(name2);
            if (encAlgFromSigAlg != null) {
                name2 = encAlgFromSigAlg;
            }
            String sigAlg = AlgorithmId.makeSigAlg(name, name2);
            try {
                SignerInfo.JAR_DISABLED_CHECK.permits(sigAlg, o);
            }
            catch (CertPathValidatorException cause2) {
                throw new SignatureException(cause2.getMessage(), cause2);
            }
            X509Certificate certificate = this.getCertificate(pkcs7);
            if (certificate == null) {
                return null;
            }
            PublicKey publicKey = certificate.getPublicKey();
            if (!SignerInfo.JAR_DISABLED_CHECK.permits(SignerInfo.SIG_PRIMITIVE_SET, publicKey)) {
                throw new SignatureException("Public key check failed. Disabled key used: " + KeyUtil.getKeySize(publicKey) + " bit " + publicKey.getAlgorithm());
            }
            if (certificate.hasUnsupportedCriticalExtension()) {
                throw new SignatureException("Certificate has unsupported critical extension(s)");
            }
            boolean[] keyUsage = certificate.getKeyUsage();
            if (keyUsage != null) {
                Object o2;
                try {
                    o2 = new KeyUsageExtension(keyUsage);
                }
                catch (IOException ex2) {
                    throw new SignatureException("Failed to parse keyUsage extension");
                }
                int booleanValue = (boolean)o2.get("digital_signature");
                int booleanValue2 = (boolean)o2.get("non_repudiation");
                if (booleanValue == 0 && booleanValue2 == 0) {
                    throw new SignatureException("Key usage restricted: cannot be used for digital signatures");
                }
            }
            Signature instance = Signature.getInstance(sigAlg);
            AlgorithmParameters parameters = this.digestEncryptionAlgorithmId.getParameters();
            try {
                SignatureUtil.initVerifyWithParam(instance, publicKey, SignatureUtil.getParamSpec(sigAlg, parameters));
            }
            catch (ProviderException | InvalidAlgorithmParameterException | InvalidKeyException cause3) {
                throw new SignatureException(cause3.getMessage(), cause3);
            }
            instance.update(array);
            if (instance.verify(this.encryptedDigest)) {
                return this;
            }
        }
        catch (IOException ex3) {
            throw new SignatureException("IO error verifying signature:\n" + ex3.getMessage());
        }
        return null;
    }
    
    SignerInfo verify(PKCS7 pkcs7) throws NoSuchAlgorithmException, SignatureException {
        return this.verify(pkcs7, (byte[])null);
    }
    
    public BigInteger getVersion() {
        return this.version;
    }
    
    public X500Name getIssuerName() {
        return this.issuerName;
    }
    
    public BigInteger getCertificateSerialNumber() {
        return this.certificateSerialNumber;
    }
    
    public AlgorithmId getDigestAlgorithmId() {
        return this.digestAlgorithmId;
    }
    
    public PKCS9Attributes getAuthenticatedAttributes() {
        return this.authenticatedAttributes;
    }
    
    public AlgorithmId getDigestEncryptionAlgorithmId() {
        return this.digestEncryptionAlgorithmId;
    }
    
    public byte[] getEncryptedDigest() {
        return this.encryptedDigest;
    }
    
    public PKCS9Attributes getUnauthenticatedAttributes() {
        return this.unauthenticatedAttributes;
    }
    
    public PKCS7 getTsToken() throws IOException {
        if (this.unauthenticatedAttributes == null) {
            return null;
        }
        PKCS9Attribute attribute = this.unauthenticatedAttributes.getAttribute(PKCS9Attribute.SIGNATURE_TIMESTAMP_TOKEN_OID);
        if (attribute == null) {
            return null;
        }
        return new PKCS7((byte[])(byte[])attribute.getValue());
    }
    
    public Timestamp getTimestamp() throws IOException, NoSuchAlgorithmException, SignatureException, CertificateException {
        if (this.timestamp != null || !this.hasTimestamp) {
            return this.timestamp;
        }
        PKCS7 tsToken = this.getTsToken();
        if (tsToken == null) {
            this.hasTimestamp = false;
            return null;
        }
        byte[] data = tsToken.getContentInfo().getData();
        CertPath generateCertPath = CertificateFactory.getInstance("X.509").generateCertPath(tsToken.verify(data)[0].getCertificateChain(tsToken));
        TimestampToken timestampToken = new TimestampToken(data);
        this.verifyTimestamp(timestampToken);
        return this.timestamp = new Timestamp(timestampToken.getDate(), generateCertPath);
    }
    
    private void verifyTimestamp(TimestampToken timestampToken) throws NoSuchAlgorithmException, SignatureException {
        String name = timestampToken.getHashAlgorithm().getName();
        if (!SignerInfo.JAR_DISABLED_CHECK.permits(SignerInfo.DIGEST_PRIMITIVE_SET, name, null)) {
            throw new SignatureException("Timestamp token digest check failed. Disabled algorithm used: " + name);
        }
        if (!Arrays.equals(timestampToken.getHashedMessage(), MessageDigest.getInstance(name).digest(this.encryptedDigest))) {
            throw new SignatureException("Signature timestamp (#" + timestampToken.getSerialNumber() + ") generated on " + timestampToken.getDate() + " is inapplicable");
        }
        if (SignerInfo.debug != null) {
            SignerInfo.debug.println();
            SignerInfo.debug.println("Detected signature timestamp (#" + timestampToken.getSerialNumber() + ") generated on " + timestampToken.getDate());
            SignerInfo.debug.println();
        }
        return;
    }
    
    @Override
    public String toString() {
        HexDumpEncoder hexDumpEncoder = new HexDumpEncoder();
        String s = "" + "Signer Info for (issuer): " + this.issuerName + "\n" + "\tversion: " + Debug.toHexString(this.version) + "\n" + "\tcertificateSerialNumber: " + Debug.toHexString(this.certificateSerialNumber) + "\n" + "\tdigestAlgorithmId: " + this.digestAlgorithmId + "\n";
        if (this.authenticatedAttributes != null) {
            s = s + "\tauthenticatedAttributes: " + this.authenticatedAttributes + "\n";
        }
        String str = s + "\tdigestEncryptionAlgorithmId: " + this.digestEncryptionAlgorithmId + "\n" + "\tencryptedDigest: \n" + hexDumpEncoder.encodeBuffer(this.encryptedDigest) + "\n";
        if (this.unauthenticatedAttributes != null) {
            str = str + "\tunauthenticatedAttributes: " + this.unauthenticatedAttributes + "\n";
            return str;
        }
        return str;
    }
    
    static {
        DIGEST_PRIMITIVE_SET = Collections.unmodifiableSet(EnumSet.of(CryptoPrimitive.MESSAGE_DIGEST));
        SIG_PRIMITIVE_SET = Collections.unmodifiableSet(EnumSet.of(CryptoPrimitive.SIGNATURE));
        JAR_DISABLED_CHECK = new DisabledAlgorithmConstraints("jdk.jar.disabledAlgorithms");
        debug = Debug.getInstance("jar");
    }
}
